@@ -173,6 +173,26 @@ class ScaleIOConnectorTestCase(test_connector.ConnectorTestCase):
             self.connector.GET_GUID_OP_CODE)
         self.get_password_mock.assert_called_once()
 
+    def test_connect_volume_old_connection_properties(self):
+        """Successful connect to volume"""
+        connection_properties = {
+            'hostIP': test_connector.MY_IP,
+            'serverIP': test_connector.MY_IP,
+            'scaleIO_volname': self.vol['name'],
+            'scaleIO_volume_id': self.vol['provider_id'],
+            'serverPort': 443,
+            'serverUsername': 'test',
+            'serverPassword': 'fake',
+            'serverToken': 'fake_token',
+            'iopsLimit': None,
+            'bandwidthLimit': None
+        }
+
+        self.connector.connect_volume(connection_properties)
+        self.get_guid_mock.assert_called_once_with(
+            self.connector.GET_GUID_OP_CODE)
+        self.get_password_mock.assert_not_called()
+
     def test_connect_volume_without_volume_id(self):
         """Successful connect to volume without a Volume Id"""
         connection_properties = dict(self.fake_connection_properties)
